#=======================================================#
#                                                       #
#   Project:    SNA for CJ                              #
#   Author:     Andrew V. Papachristos                  #
#   Copyright:  2015                                    #
#  													                            #
#	  Module:     Analytics										            #
#														                            #
#   Tasks:      Setting up RStudio Session				      #
#				        Importing Edgelist and Attribute Data   #
#               Setting Up the Social Network           #  
#               Density                                 #
#               Components                              #
#               Degree                                  #
#               Cut-points                              #
#               K-core                                  #
#               Distance                                #
#               Brokerage/ Betweenness                  #
#               Neighborhoods                           #
#               Compiling Network Statistics            #
#               Review                                  #
#                                                       #
#=======================================================#

#--------------------------------------------------------------------

#   1.    Setting up RStudio Session 

# Congratulations on setting your working directory and opening the analytics.R script file.

# Check the contents of the working directory using the dir() command. Printed in the Console pane below will be a list of all the items available in the working directory. You should see the name of this script as well as the 2 csv files: arrests.csv and attributes.csv.

dir()

# Open the igraph library. 

library(igraph)





#--------------------------------------------------------------------

#   2.    Importing Edgelist and Attribute Data

# Load the arrests.csv edgelist file from the Analytics RStudio Files folder using the read.csv() command. Name the RStudio object for this csv "e".

e <- read.csv("arrests.csv", header=FALSE)

# Inspect the new object called "e" to check how the csv imported. 

e

# The object "e" contains 25 rows and 2 columns of names. Each row represents an undirected edge in the network. The numbers 1-25 to the left of each row are row identifiers and are not internal to the csv. 

# Next, use the read.csv() command to load the attribute data contained in attributes.csv. Name the RStudio object for this csv "a".

a<-read.csv("attributes.csv", header=TRUE)

# Inspect the new object called "a" to check how the csv imported. 

a

# The object "a" contains a header row of first names, gang names, gang id numbers, and gender. The column on the far left prints a row number for each row. There are 20 rows, and each row contains attribute information on 1 of the 20 nodes in the network. 





#--------------------------------------------------------------------

#   3.    Setting Up the Social Network

# Co-arrests are not directed events, so there is no need to distinguish between sender and receiver. For purposes of this lab, this assumption introduces simpler calculations of network analytics. Create the network from the edgelist and match the attribute data to the correct nodes using the graph.data.frame() command. Call the new network object "g". To review this command return to Module 2: Data RStudio Lab.

g <- graph.data.frame(e, directed=FALSE, vertices=a)

# Inspect the new network object "g". 

g

# As we can see from our new network, "g" has 20 nodes and 25 edges. There are 4 attributes for the nodes included in the network: name, gang, gang_id, and gender.

# Inspect the plot of network "g". The customization of the plot() command sets the color of the nodes to vary by gang. The first names are the default labels for the nodes because the names were used to build the edgelist. If the command below is confusing, return to the Module 3: Visualization RStudio Lab for a refresher.

plot(g, layout=layout.kamada.kawai, vertex.size=20, vertex.label.color="white", vertex.label.cex=.7, vertex.color=V(g)$gang_id, edge.width=3)

# Remember to click on the Zoom button in the lower right Plots pane to see a larger version of your plot. Now the network is set up, and we have a sense of what the network looks like based on the plot. Next, we will walk through a series of simple network statistics available in the igraph library. All of the statistics below were introduced in the Module 4: Analytics Tutorial.





#--------------------------------------------------------------------

#   4.    Density

# Density measures the number of edges in a network as a ratio of the all of the possible edges of a network. Dividing the number of present ties by the number of possible ties equals a network's density. The density command in igraph is graph.density(). For more information on the graph.density() command, access the help menu with the following command. 

?graph.density

# Run the following graph.density() command to obtain the single value for the density of the entire network "g".

graph.density(g)

# The result of 0.1315789 means that 13.16% of all possible ties are present in network "g".





#--------------------------------------------------------------------

#   5.    Components

# Components describe how connected a network is or how many separate pieces there are within a single network. Components are discrete sections of the network. Sometimes we are only interested in analyzing the largest component, the discrete portion of the network that connects the largest number of nodes. As you can see in the plot of network "g", not all of the nodes in this network are connected within the same component. Instead, the nodes are spread across 3 different components. In larger networks, it is sometimes difficult to quickly count all of the components. The clusters() command in igraph provides a way to generate information about all of the components in small or large networks. For more information on the clusters() command, access the help menu. 

?clusters

# To produce a count of the number of components in the network "g" add the $no (i.e., number) to the end of the clusters() command for network "g". Remember that the dollar sign is an extraction device in RStudio. Because the clusters() command returns several pieces of information about components, we use the dollar sign to extract particular results of the command.

clusters(g)$no

# You should see the answer of 3 in the Console pane below. This means that there are three components in network "g". We were able to confirm this visually in our plot of network "g".  The command to list the number of components might seem obvious with this small sample network "g", but if we had a large network with hundreds of components, this command would save a lot of time. 

# Extract the $csize (i.e., component size) information from the clusters() command to generate a list of the size of each component.

clusters(g)$csize

# The results show that the 1st component contains 6 individuals, the 2nd component contains 2 individuals, and the 3rd component contains 12 individuals. This information shows us that the 3rd component is the largest component in this network because it connects the most nodes.

# Extract the $membership (i.e., which component does each node belong) information from the clusters() command to generate an ordered list of component id numbers for each node in the network.

clusters(g)$membership

# The results for clusters(g)$membership produce an ordered list of 20 component membership id numbers. Each component membership id number matches a node in the network. To see this result, use the table() command for the first names and the component membership.  

table(V(g)$name,clusters(g)$membership)

# Each of the 3 components becomes a column in the table. Zeros indicate no membership in that component, and a 1 indicates membership in that component. It is impossible to have membership in multiple components because components are discrete. Now we can see the names of the nodes in each of the three components.

# In the previous plot, we used gang membership to vary the color of the nodes. Use the plot() command below to vary the color of the nodes by component membership. 

plot(g, layout=layout.kamada.kawai, vertex.size=20, vertex.label.color="white", vertex.label.cex=.7, vertex.color=clusters(g)$membership, edge.width=3)

# One thing to remember about components is that if we had a network with many of isolates (nodes with zero ties to other nodes), each isolate would count as a separate component. The example arrest data here have no isolates.





#--------------------------------------------------------------------

#   6.    Degree

# Calculating the degree for an undirected network is straightforward--degree equals the number of ties for each node. It is the counting of the number of lines connected to each dot. Use the command below to access the help menu for more information on the igraph command for degree.

?degree

# Use the degree() command below to make a list of each node's degree. There are 20 nodes in this network and 20 degree scores. Notice that you are not creating a new object with this command. Nothing new appears under the Data section in your Environment pane. This is just a print command to get results in the Console pane.

degree(g)

# Make a histogram to show the degree distribution for network "g". 

hist(degree(g))

# Or use base functions in RStudio to calculate the mean, standard deviation, minimum, and maximum values for degree. Notice the double parentheses required to execute these commands.

mean(degree(g))
sd(degree(g))
min(degree(g))
max(degree(g))

# Degree is also useful for identifying isolates in your data. If degree is equal to zero then those nodes are not connected to any others in the network. It would be up the researcher to decide how to use or treat isolates in the analysis.

# As we did with components above, we can use the degree as part of the network visualization. First, let's generate the same ordered list of degree scores for each node, but this time we will save it as an attribute called degree within network "g". Remember that the uppercase V tells RStudio to save the list of degrees as a vertex (node) attribute within network "g". 

V(g)$degree <- degree(g)

# You can inspect the names or labels of the vertex attributes in network "g" by entering just the name of the network. You should see the new degree attribute included in the list under "attr:" which is short for attributes.

g

# The plot() command below varies the size of the nodes by the degree score using the new attribute that we just created above. Adjusting the vertex size also includes a multiplier of 5 that is applied evenly to all nodes in order to avoid the nodes getting too small. Try running the plot command with and without the "*5" multiplier that is included after degree for vertex size.

plot(g, layout=layout.kamada.kawai, vertex.size=V(g)$degree*5, vertex.label.color="black", vertex.label.cex=.7, vertex.color="lightblue", edge.width=3)

# Degree can be used to generate theoretical tests. What would the network look like if we removed the node with the largest degree? Use the maximum command, max(), to identify the largest degree score in the network. We did this earlier, but here is the command again.

max(degree(g))

# If you add the table() command to the beginning of the maximum command, you can see how many people have the maximum degree of 6.

table(max(degree(g)))

# Only 1 person has the maximum degree of 6. We can see in the plot that this hypothetical person's name is Kevin. Or we can use the command below that asks RStudio what is the first name of the person in network "g" who has a degree equal to 6.

V(g)$name[V(g)$degree==6]

# Now that we know which node has the highest degree, we can delete this node from the network to test its effects. Below we use the delete.vertices() command to remove Kevin with the highest degree. Because we don't want to change anything in our original network "g", we create a new network called "g2" using the assignment arrow. This will create a new network called "g2" without Kevin and his ties, and our original network "g" will not be affected. 

g2<-delete.vertices(g, which((V(g)$degree)==6))

# Inspect the new network g2 and note that this network has only 19 nodes and 19 ties.

g2

# Inspect the new network g2 using the plot command.

plot(g2, layout=layout.kamada.kawai, vertex.size=V(g2)$degree*5, vertex.label.color="black", vertex.label.cex=.7, vertex.color="lightblue", edge.width=3)

# The network without Kevin looks very different. Remember that you can use the blue arrows in the Plots pane to scroll through previous versions of plots, and click on Zoom if the Plots pane is too small to see the detail. Most noticeable in the network without Kevin is that there are more components than before. Do you recall the commands introduced in the section above that generat counts and sizes of the components? Let's use those commands to inspect our new "g2" network that is missing the highest degree node.

clusters(g2)$no
clusters(g2)$csize

# How many components are there without Kevin? Are there isolates in the "g2" network? What is the size of the new largest component? We can answer these questions with the component commands above. 





#--------------------------------------------------------------------

#   7.    Cut-points

# Essentially what we did by removing Kevin from the network in section 6 on degree was locate a cut-point. Cut-points are nodes that when removed from the network increase the number of components in the network. Kevin was a cut-point because there were 3 components in the original network and when Kevin was removed from the network there were 6 components. Removing Kevin and his ties fragmented the network. For more information on the igraph command for cut-points access the help menu using the following command.

?articulation.points

# The code below identifies which nodes are cut-points in network "g". This command will show the node identification numbers in the Console pane.

articulation.points(g)

# Now we know that the 6th, 17th, 20th, and 12th individuals in network "g" are cut-points. By pulling up the list of names in the network, we can see that the individuals with those id numbers are Robert, Kevin, Michael, and Julie.

V(g)$name

# Let's make a new cut-point variable that sets cut-points to the color red. Then we can use cut-points in plotting and deletion techniques.

g<-set.vertex.attribute(g, "cut.point", articulation.points(g), "red")

# Inspect the new cut-point variable. Are numbers 6, 12, 17, and 20 different than the others in the list? "NA" stands for "not available" and identifies missing values within variables.  

V(g)$cut.point

# Plot the network "g," and vary the color of the nodes to distinguish the cut-points.

plot(g, layout=layout.fruchterman.reingold, vertex.size=25, vertex.color=V(g)$cut.point, vertex.label.color="black", vertex.label.cex=.7, edge.width=3)

# What do you notice about the location of the cut-points in the plot? Are cut-points necessarily nodes with the highest degree?

# Let's delete the nodes that are cut-points from the network, but let's do this is a new network so we don't change anything in our original network "g". We will create a new network object called "g3", which will be the same as network "g" but without the cut-points and their ties. 

g3 <- delete.vertices(g, which(V(g)$cut.point=="red"))

# Inspect the new "g3" network. Note that it has only 16 individuals since we deleted 4 individuals as cut-points from the original 20. It also has only 12 ties compared to the 25 ties in network "g".

g3

# Inspect the plot of the new network using the basic plot() command.

plot(g3)

# What was the result of removing the cut-points? How many components are in the new network "g3"? How many isolates are in the new network "g3"? What is the new distribution of degree scores? Use the space below to recall some of the commands introduced earlier in this script that can be used to answer these questions.





#--------------------------------------------------------------------

#   8.    K-core

# K-core identifies areas in the network where nodes have similar degrees. Often k-core identifies completely connected subgroups. K-core measures the minimum degree of nodes in a particular connected area of the network. The igraph command for k-core is graph.coreness(). For more information access help on this command.

?graph.coreness

# Use the graph.coreness() command below to get a list of each node's k-core. 

graph.coreness(g)

# We see that John, Joshua, William, and Julie have the largest k-core of 3. Each of these nodes in this area of the network has a degree of at least 3. If we look at the basic plot of "g", we can see that these 4 nodes are all connected to each other and they all have at least three ties.

plot(g)

# We can customize the plot to highlight the differences in k-core. The colors and labels identify the different sizes of maximum subgroups.

plot(g, layout=layout.fruchterman.reingold, vertex.color=graph.coreness(g), vertex.label=graph.coreness(g), vertex.label.color="white")





#--------------------------------------------------------------------

#   9.    Distance

# Distance is the number of steps it takes for two nodes to reach each other along the most efficient path. Identify two nodes in the largest component of the plot for "g". Identify all of the possible paths that could connect those two nodes. The number of ties or steps of the shortest possible path between two nodes is the geodesic distance. Distance is a limited measure in that you can only calculate distance within a single component. How would you explain, for example, the distance between Michelle and Wes? Michelle is in the one dyad in this network and Wes is in the mid-sized component of 6. There is no possible path between them, so the geodesic distance between them is undefined. For this part of the lab on distance, we will limit our analysis to the largest component. First we need to create a new network object the deletes all nodes and ties that are not part of the largest component. First let's remind ourselves which component is the largest.

clusters(g)

# The 3rd component is the largest. The extraction of $csize shows the size of each component, and 12 is larger than 6 and 2. We can delete vertices and their edges if they have a component membership equal or less than 2. That will leave us only with those in the 3rd and largest component. We will create a new network object called "g4" to avoid altering our original network "g".

g4 <- delete.vertices(g, which(clusters(g)$membership<=2))

# Inspect this new network containing just the largest component.

g4
plot(g4)

# Our largest component network "g4" contains 12 nodes and 17 edges that are all connected within a single component. The geodesic distance measure in igraph uses a command called shortest.paths().

?shortest.paths

# Run the shortest.paths() command below to obtain a matrix that calculates the shortest distance between all of the nodes in the largest component.

shortest.paths(g4)

# The matrix might not fit cleanly in the Console pane below, but all of the information is available by scrolling up and down in the Console pane. Notice that along the diagonal is a series of zeros. This is because nodes do not have geodesic distances to themselves, only to others. Looking at the matrix, can you tell which nodes have the longest distance between them? How many steps must those farthest from each other travel to reach each other? Remember that the diameter of a network is the longest geodesic distance. We can identify the diameter by looking through the matrix of distances or by using the maximum command.

max(shortest.paths(g4))

# We might be less interested in each path between all possible combinations of nodes and more interested in an overall summary of who has the farthest to travel to reach everyone in the network versus who has the shortest to travel. In order to do the latter, we need to take an average of each column or each row (since the rows and columns are identical in a geodesic distance matrix of undirected networks). Calculating the average for each column or row will produce each node's average distance to everyone else in the network. Below, we use the colSums() command that sums each column, and then we divide the sum of each column by 12-1. There are 12 people in the largest component, but we subtract 1 because we aren't interested in including the distance to oneself that is included in each column along the diagonal of zeros in the matrix. The minus 1 ensures that we calculate a mean of actual possible paths. To the left of the assignment arrow, we tell RStudio to save this information as a node attribute of network "g4", and we call the attribute "mean.distance." Quite a few parentheses are required on the right of the assignment arrow to preserve the correct order of operations.

V(g4)$mean.distance <- (colSums(shortest.paths(g4)))/(12-1)

# Now each of the 12 nodes in the largest component has a value of its average geodesic distance--an average of all 11 of the shortest paths to the 11 other nodes in the component. 

V(g4)$mean.distance

# We can inspect this calculation manually. For example, node #1 is John and according to the mean.distance variable his average geodesic distance is 3.0. If we look at the matrix printed in the Console we can see each of John's geodesic distances in the 1st column of numbers (or the 1st row of numbers). If we add all of these together ignoring the zero and divide by 11, we should get 3.0.

(1+1+1+4+4+4+5+3+4+4+2)/11

# With the information on each node's geodesic distance no longer in matrix form, it is easy to identify an overall average of distance for the largest component, the standard deviation from that average, who has the longest average distance, and who has the shortest average distance. 

mean(V(g4)$mean.distance)             # mean distance of the component
sd(V(g4)$mean.distance)               # standard deviation of that mean
max(V(g4)$mean.distance)              # value of maximum average distance
min(V(g4)$mean.distance)              # value of minimum average distance
V(g4)$name[(V(g4)$mean.distance)>=3.18]  # Which node has the max distance?
V(g4)$name[V(g4)$mean.distance<=1.73]    # Which node has the min distance?

# Are you surprised that Harry has the longest average to reach everyone else in the network? Are you surprised that Kevin has the shortest average to reach everyone else in the network? Can you see why Michael's average is just a bit larger than Kevin's?





#--------------------------------------------------------------------

#   10.    Brokerage/ Betweenness

# It is important to understand distance before introducing brokerage. Remember all of those shortest paths we calculated in the geodesic distance section above? Along many of those paths are other nodes that are important in maintaining the shortest paths. We can measure the total number of geodesic distances that pass through each node, which is called a betweenness score. Betweenness is one way to measure brokerage. The igraph command is betweenness(). It counts the number of geodesic distances that go through each node. 

?betweenness

# We had to limit our analysis to a single component when we looked at distance. We chose to examine the largest component, which we assigned to the network object "g4". The same requirement is true for analyzing betweenness. In order to count the shortest paths that go through each node, we need to use a single component so that the shortest paths are definable.  

betweenness(g4)

# A high betweenness score means that more geodesic distnces go through that node, and that node acts as a broker in the network. A low betweenness score means that few geodesic distances go through that node, and that node is not a broker in the network. Which node is not brokering any distances in this component? Who has the highest betweenness score, or who is the most serious broker in this component? Can you see why Daniel and James have higher betweenness scores than Charlie and Scott?

# Let's save betweenness as an attribute of nodes in network "g4" so that we can include it in our plot customizations. 

V(g4)$betweenness <- betweenness(g4)

# Customize the size of the nodes by their brokerage score. 

plot(g4, layout=layout.fruchterman.reingold, vertex.size=V(g4)$betweenness, vertex.label.color="black")





#--------------------------------------------------------------------

#   11.    Neighborhoods

# Thinking about neighborhoods is somewhat like thinking about distances, but with neighborhoods we are usually interested in those nodes closest to a particular node of interest rather than the paths to every other node in the network. Imagine that we need to identify all of the direct associates of a single node (i.e., 1st order neighborhood) or alternatively for good measure we want all of Daniel's associates plus his associates' associates as well (i.e., 2nd order neighborhood). Neighborhoods assume that further removed associates might not be as important, and so we don't need to include them in our examination. Let's say Daniel is in some trouble. We want to identify Daniel's friends but we might also want to identify Daniel's friends plus the friends' friends. Daniel's 1st order neighborhood includes him and just his friends or the nodes connected directly to Daniel. Daniel's 2nd order neighborhood includes Daniel, all of the 1st order (his direct friends), plus everyone directly connected to his friends. In other words, Daniel's 2nd order neighborhood includes all the nodes that are within 2 steps or 2 ties from Daniel. In order to do a neighborhood analysis for Daniel, first we need to identify which number Daniel is in our network "g".

V(g)$name

# From the list of names we can see that Daniel is #15 in the network. To identify Daniel's neighborhood, we will use the different neighborhood() commands in igraph.

?neighborhood

# Let's identify the size of Daniel's 2nd order neighborhood using the neighborhood.size() command. In the parentheses we need to include 3 pieces of information, and each piece needs to be separated by commas. The first piece of information is the name of the network, "g", which has been similar to the majority of the commands used in this lab. The second piece of information is the order of the neighborhood that we want to return. Because we want Daniel's friends and friends of friends then we need to specify that we want Daniel's 2nd order neighborhood. The third piece of information identifies which node is the ego or target of the neighborhood. We know that Daniel is node #15, so the number 15 is entered as the 3rd piece of information in the parentheses below. Alternatively, if we did not know Daniel's node number, we could use the node attribute command equal to the name "Daniel" and obtain the same results. Both examples are included below.

neighborhood.size(g, 2, 15)
neighborhood.size(g, 2, V(g)$name=="Daniel")

# Daniel's 2nd order neighborhood includes 8 nodes. Here is a list of node identification numbers that are included in Daniel's 2nd order neighborhood.

neighborhood(g, 2, 15)

# Now let's create a new network that includes only Daniel as the target node and his 2nd order neighborhood. We will call this network object "g5".

g5 <- graph.neighborhood(g, 2, 15)

# Inspect the new network "g5".

g5

# "g5" is similar to many of the networks that we have created up to this point with one small exception. Note the [[1]] at the top of the information printed below in the Console pane. We will need to remember this piece of the network object when we use the plotting command.

plot(g5[[1]])

# This plots includes only Daniel's 2nd order neighborhood. We see that the 8 nodes in Daniel's 2nd order neighborhood include his own node, his 3 friends, and 4 friends of his friends. 

# If we want to color code this 2nd order neighborhood to show distance from Daniel, we will need to back up a little. Let's assign Daniel, our target, the color red, the friends in his 1st order neighborhood orange, and the additional nodes of his 2nd order neighborhood yellow. First, create a new attribute variable called "color" and assign all 20 nodes "none" for the color. 

V(g)$color <- "none"

# Replace Daniel's node to equal the color red.

V(g)$color[V(g)$name=="Daniel"] <- "red"

# Generate the list of id numbers for the members of Daniel's 1st order neighborhood.

neighborhood(g, 1, 15)

# Daniel is node 15, and we want his node to stay red. In our plot, we want nodes 16, 17, and 18 to be orange.

V(g)$color[16:18] <- "orange"

# Now we need the id numbers for the nodes that get added in the 2nd order neighborhood. The list will produce those in the first order as well, so we will just look at those numbers that we haven't assigned a color yet. 

neighborhood(g, 2, 15)

# Nodes with the id numbers 13, 14, 19, and 20 need to be yellow.

V(g)$color[c(13, 14, 19, 20)] <- "yellow"

# Let's see how we did with these color assignments.

V(g)$color

# Now we can add this customization to the plot command. First, we need to subset the whole network "g" again to just the target and the 2nd order neighborhood in order to copy over the color variable we just made.

g5 <- graph.neighborhood(g, 2, 15)

plot(g5[[1]], vertex.color=g5[[1]]$color, vertex.size=45, vertex.label.color="black", edge.width=3)





#--------------------------------------------------------------------

#   12.    Compiling Network Statistics

# Having completed all of these analyses, it might be the case that you want these network statistics in a table format that can be used outside of RStudio. We can make tables and export them as .csv files in RStudio so that they are compatible with other software like Microsoft Excel or Open Office. Let's make a table called "network.analytics". This table name goes on the left of the assignment arrow. Use the data.frame() command on the right of the assignment arrow. The data.frame() command combines specified variables into a table. Essentially, we want to take several lists that are all the same size and merge them together as separate columns. This command will help preserve the order of each item in each column so that the values match up with the correct nodes. In the parentheses following the data.frame() command, we include all of the variables of interest that we generated for the full sample of 20 nodes. 

network.analytics <- data.frame(V(g)$name, clusters(g)$membership, V(g)$degree, V(g)$cut.point, graph.coreness(g), V(g)$color)

# Add names to the columns using the colnames() command.

colnames(network.analytics) <- c("name", "component", "degree", "cut.point", "k.core", "Daniel.neighborhood")

# Inspect the new table either by running just the name of the new object, or click on the "network.analytics" object under data in the Environment pane.

network.analytics

# Use the write.table() command to save the table in a format that can be used outside of RStudio. The option "col.names=" is set to "NA" because this aligns the column names with the appropriate column leaving a blank cell for the row names that were automatically generated using the data.frame() command. 

write.table(network.analytics, file="network_analytics.csv", sep=",", col.names=NA)

# The csv file will appear in the working directory, which we have set to the Analytics RStudio Files folder. Go to that folder on your computer now and open the new file you just created. 

# We have several analytics that are only available for the nodes in the largest component. For those we will create a second table using the data.frame() command.

largest.component <- data.frame(V(g4)$name, V(g4)$mean.distance, V(g4)$betweenness)

# Attach column names. 

colnames(largest.component) <- c("name", "mean.distance", "betweenness")

# Inspect the table by running the command below or by clicking on the largest.component object in the Environment pane.

largest.component

# Use the write.table() command to save this table as a .csv file in the working directory.

write.table(largest.component, file="largest_component_analytics.csv", sep=",", col.names=NA)

# Inspect the .csv file in the working directory now. 





#--------------------------------------------------------------------

#   13.    Review

# Below is a summary list of the commands introduced in the Module 4: Analytics RStudio Lab. You can use these commands on other networks or review the commands and the analytics here without the full commentary in the lab above. 

graph.density(g)                # density of the network
clusters(g)$no                  # number of components
clusters(g)$csize               # the size of each component
clusters(g)$membership          # a component membership list for all nodes
degree(g)                       # a list of degrees of all nodes
articulation.points(g)          # identify the cut-points
graph.coreness(g)               # a list of k-cores for all nodes

shortest.paths(g4)              # a matrix of distances within a component
betweenness(g4)                 # a list of betweenness scores 

neighborhood.size(g, 2, 15)     
                  # number of nodes in 2nd order neighborhood for node #15
neighborhood(g, 2, 15)
                  # the ids for nodes in 2nd order neighborhood for node #15





#-END----------------------------------------------------------------